#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include "vtty_db.h"

#define VTTY_VERSION "1.0"
#define VTTY_MASTER_PREFIX "/dev/vpty"
#define VTTY_SLAVE_PREFIX "/dev/tty"
#define VTTY_MAJOR_MASTER 226
#define VTTY_MAJOR_SLAVE 227
#define VTTY_TTYD_PATH "/usr/local/bin/ttyd"
static int toffering; // Telnet Options Offering - JOJ

enum {
	VTTY_CMD_ADD = 1,
	VTTY_CMD_ENABLE = 2,
	VTTY_CMD_DISABLE = 3,
	VTTY_CMD_REMOVE = 4,
	VTTY_CMD_STATUS = 5,
	VTTY_CMD_INIT = 6
};

const unsigned long brate[] = {
	B75,
	B150,
	B200,
	B300,
	B600,
	B1200,
	B1800,
	B2400,
	B4800,
	B9600,
	B19200,
	B38400,
	B57600,
	B115200,
	B230400
};

struct _pset_t 
{
	char *symbol;
	int val;
};

const struct _pset_t pset_db[] = {
	{"8", DB8},
	{"7", DB7},
	{"6", DB6},
	{"5", DB5},
	{NULL, 0}
};

const struct _pset_t pset_parity[] = {
	{"N", PARITY_NO},
	{"E", PARITY_EVEN},
	{"O", PARITY_ODD},
	{NULL, 0}
};

const struct _pset_t pset_fc[] = {
	{"C0", FC_NOHW},
	{"C1", FC_HW},
	{"S0", FC_NOSW},
	{"S1", FC_SW},
	{NULL, 0}
};

struct _vtty_command_set
{
	int command;
	struct _vtty_db_type_t type;
};

void print_usage(char *progname)
{
	fprintf (stderr,
		"vtty version %s, 2003 Sena Technologies, Inc.\r\n"
		"Usage:\r\n"
		"  %s { add | addnt } TTYSUFFIX ip_address tcp_port [serial_option]\r\n"
		"  %s { enable | enablent | disable | remove } TTYSUFFIX\r\n"
		"  %s status [TTYSUFFIX]\r\n"
		"\r\n"
		"  serial_option : -b <baudrate> -p <port_setting>\r\n"
		"    <baud_rate>    : one of the following\r\n"
		"                75, 150, 200, 300, 600, 1200, 1800, 2400, 4800,\r\n"
		"                9600, 19200, 38400, 57600, 115200, 230400\r\n"
		"    <port_setting> : specified by a string of one or more of\r\n"
		"                     the following concatenated together with\r\n"
		"                     no intervening spaces\r\n"
		"                8  : 8 bits / Character\r\n"
		"                7  : 7 bits / Character\r\n"
		"                6  : 6 bits / Character\r\n"
		"                5  : 5 bits / Character\r\n"
		"                N  : No parity\r\n"
		"                E  : Even parity\r\n"
		"                O  : Odd parity\r\n"
		"                C0 : No hardware flow control\r\n"
		"                C1 : Hardware flow control\r\n"
		"                S0 : No software flow control\r\n"
		"                S1 : Software flow control\r\n"
		"  If the serial option is not specified, the serial option\r\n"
		"  defaults to : baudrate = 9600, port setting = 8NC0\r\n"
		"\r\n"
		"Example:\r\n"
		"  %s add mysuffix 192.168.1.1 6001 -b 9600 -p 8NC0\r\n"
		"  %s enable mysuffix\r\n"
		"  %s status\r\n",
		VTTY_VERSION, progname, progname, progname, progname, progname, progname);
}

static int check_ip(char *ip)
{
	int a=-1, b=-1, c=-1, d=-1;
	sscanf(ip, "%d.%d.%d.%d", &a, &b, &c, &d);
	if(a < 0 || a > 255) return -1;
	if(b < 0 || b > 255) return -1;
	if(c < 0 || c > 255) return -1;
	if(d < 0 || d > 255) return -1;
	return 0;
}

static int check_baudrate(char *cbrate, unsigned long *baudrate)
{
	int i=0, ret = -1;
	while(brate[i]) {
		if(brate[i] == atol(cbrate)) {
			*baudrate = brate[i];
			return 0;
		}
		i++;
	}
	return ret;
}

static int check_port_setting(char *cpset, int *databits, int *parity, int *flowctrl)
{
	int i=0, ret = -1;
	char *p = cpset;

	// default value;
	*databits = DB8;
	*parity = PARITY_NO;
	*flowctrl = FC_NOHW;

	while(pset_db[i].symbol) {
		if(strstr(p, pset_db[i].symbol)) {
			*databits = pset_db[i].val;
			break;
		}
		i++;
	}

	i=0;
	while(pset_parity[i].symbol) {
		if(strstr(p, pset_parity[i].symbol)) {
			*parity = pset_parity[i].val;
			break;
		}
		i++;
	}
	
	i=0;
	while(pset_fc[i].symbol) {
		if(strstr(p, pset_fc[i].symbol)) {
			*flowctrl = pset_fc[i].val;
			break;
		}
		i++;
	}
	return 0;
}

int parse_options (int wc, char *words[], struct _vtty_command_set *cmd)
{
	int i;
	int bargnum = 1;

	if(!strcmp(words[1], "add"))
	{
		cmd->command = VTTY_CMD_ADD;
                toffering = 1; // Telnet Options Offering - JOJ
		if(wc < 5) bargnum = 0;
	}
	else if(!strcmp(words[1], "addnt"))//Add w/o Telnet Option Offering-JOJ
	{
		cmd->command = VTTY_CMD_ADD;
                toffering = 0; // Telnet Options Offering - JOJ
		if(wc < 5) bargnum = 0;
	}
	else if(!strcmp(words[1], "enable"))
	{
		cmd->command = VTTY_CMD_ENABLE;
                toffering = 1; // Telnet Options Offering - JOJ
		if(wc < 3) bargnum = 0;
	}
	else if(!strcmp(words[1], "enablent"))
	{
		cmd->command = VTTY_CMD_ENABLE;
                toffering = 0; // Telnet Options Offering - JOJ
		if(wc < 3) bargnum = 0;
	}
	else if(!strcmp(words[1], "disable"))
	{
		cmd->command = VTTY_CMD_DISABLE;
		if(wc < 3) bargnum = 0;
	}
	else if(!strcmp(words[1], "remove"))
	{
		cmd->command = VTTY_CMD_REMOVE;
		if(wc < 3) bargnum = 0;
	}
	else if(!strcmp(words[1], "status"))
	{
		cmd->command = VTTY_CMD_STATUS;
	}
	else if(!strcmp(words[1], "init"))
	{
		cmd->command = VTTY_CMD_INIT;
	}
	else
	{
		fprintf(stderr, "\r\n%s - Uknown command.\r\n\r\n", words[1]);
		print_usage(words[0]);
		return -1;
	}
	
	if(bargnum == 0)
	{
		fprintf(stderr, "\r\nInsufficient arguments.\r\n\r\n");
		print_usage(words[0]);
		return -1;
	}

	switch(cmd->command)
	{
		case VTTY_CMD_ADD:
			for(i = 2 ; i < wc ; )
			{
				if(i == 2)
				{
					if(strlen(words[i]) >= MAXLEN_TTYSUFFIX)
					{
						fprintf(stderr, "%s - Too long TTYSUFFIX. \r\nThe maximum length is %d\r\n", words[i], MAXLEN_TTYSUFFIX-1);
						return -1;
					}
					sprintf(cmd->type.ttysuffix, "%s", words[i]);
					i++;
				}
				else if(i == 3)
				{
					if(check_ip(words[i]) < 0)
					{
						fprintf(stderr, "%s - Invalid ip address.\r\n", words[i]);
						return -1;
					}
					sprintf(cmd->type.target_ip, "%s", words[i]);
					i++;
				}
				else if(i == 4)
				{
					int port = -1;
					port = atoi(words[i]);
					if(port <= 0 || port > 65535)
					{
						fprintf(stderr, "%s - Invalid TCP port.\r\n", words[i]);
						return -1;
					}
					cmd->type.target_port = port;
					i++;
				}
				else if(i > 4 && i < 9 && words[i][0] == '-' && words[i+1])
				{
					unsigned long baudrate = 0;
					int databits = 0, parity = 0, flowctrl = 0;
					switch(words[i][1])
					{
						case 'b':
							if(check_baudrate(words[i+1], &baudrate) < 0)
							{
								fprintf(stderr, "%s - Invalid baudrate.\r\n", words[i+1]);
								return -1;
							}
							cmd->type.uart.baudrate = baudrate;
							i += 2;
							break;
						case 'p':
							if(check_port_setting(words[i+1], &databits, &parity, &flowctrl) < 0)
							{
								fprintf(stderr, "%s - Invalid port setting.\r\n", words[i+1]);
								return -1;
							}
							cmd->type.uart.databits = databits;
							cmd->type.uart.parity = parity;
							cmd->type.uart.flowctrl = flowctrl;
							i += 2;
							break;
						default:
							i++;
							break;
					}
				}
				else
					break;
			}
			break;
		case VTTY_CMD_ENABLE:
		case VTTY_CMD_DISABLE:
		case VTTY_CMD_REMOVE:
		case VTTY_CMD_STATUS:
			for(i = 2 ; i < wc ; )
			{
				if(i == 2)
				{
					if(strlen(words[i]) >= MAXLEN_TTYSUFFIX)
					{
						fprintf(stderr, "%s - Too long TTYSUFFIX. \r\nThe maximum length is %d\r\n", words[i], MAXLEN_TTYSUFFIX-1);
						return -1;
					}
					sprintf(cmd->type.ttysuffix, "%s", words[i]);
					i++;
				}
				else
					break;
			}
			break;
		case VTTY_CMD_INIT:
			break;
	}
	return 0;
}

int print_status(struct _vtty_db_type_t *data, int net_state)
{
	int COL_SIZE[6] = { 7, 24, 16, 6, 12, 14 };
	int index = 0, i=0, j=0;
	char buff[78] = {0, };
	char temp[78] = {0, };
	memset(&buff[0], ' ', sizeof(buff)-1);

	// minor
	bzero(&temp[0], sizeof(temp));
	sprintf(temp, "   %03d", data->minor);
	memcpy(&buff[0]+index, &temp[0], strlen(temp)); 
	// tty 
	index += COL_SIZE[i++];
	bzero(&temp[0], sizeof(temp));
	snprintf(temp, COL_SIZE[1]-1, "%s%s", VTTY_SLAVE_PREFIX, data->ttysuffix);
	memcpy(&buff[0]+index, &temp[0], strlen(temp)); 
	// ip 
	index += COL_SIZE[i++];
	bzero(&temp[0], sizeof(temp));
	snprintf(temp, COL_SIZE[2]-1, "%s", data->target_ip);
	memcpy(&buff[0]+index, &temp[0], strlen(temp)); 
	// port
	index += COL_SIZE[i++];
	bzero(&temp[0], sizeof(temp));
	snprintf(temp, COL_SIZE[3]-1, "%d", data->target_port);
	memcpy(&buff[0]+index, &temp[0], strlen(temp)); 
	// serial 
	index += COL_SIZE[i++];
	bzero(&temp[0], sizeof(temp));
	sprintf(temp, "%d-", data->uart.baudrate);
	while(pset_db[j].symbol) {
		if(data->uart.databits == pset_db[j].val) {
			strcat(temp, pset_db[j].symbol); break;
		} j++;
	}
	j=0;
	while(pset_parity[j].symbol) {
		if(data->uart.parity == pset_parity[j].val) {
			strcat(temp, pset_parity[j].symbol); break;
		} j++;
	}
	j=0;
	while(pset_fc[j].symbol) {
		if(data->uart.flowctrl == pset_fc[j].val) {
			strcat(temp, pset_fc[j].symbol); break;
		} j++;
	}
	memcpy(&buff[0]+index, &temp[0], strlen(temp)); 
	// status 
	index += COL_SIZE[i++];
	bzero(&temp[0], sizeof(temp));
	if(data->benable == STATUS_ENABLED)
	{
		if(net_state >= 0)
			snprintf(temp, COL_SIZE[5]-1, "CONNECTED");
		else
			snprintf(temp, COL_SIZE[5]-1, "DISCONNECTED");
	}
	else
		snprintf(temp, COL_SIZE[5]-1, "DISABLED");
	memcpy(&buff[0]+index, &temp[0], strlen(temp)); 

	printf("%s\r\n", buff);
}

int check_network(const char *ip, int port)
{
#define VTTY_NET_ESTABLISHED "ESTABLISHED"
	FILE *fp;
	char portstr[20] = {0, };

	fp = popen("netstat -t", "r");
	sprintf(portstr, "%d", port);
	if(fp)
	{
		char buff[512] = {0, };
		while(fgets(buff, sizeof(buff), fp))
		{
			if(strstr(buff, ip) != NULL && strstr(buff, portstr) != NULL && strstr(buff, VTTY_NET_ESTABLISHED) != NULL)
				return 0;
		}
		fclose(fp);
	}
	return -1;
}

int get_ttyd_pid(char *ttysuffix)
{
	FILE *fp;
	char cmd[MAXLEN_TTYSUFFIX+128] = {0, };
	char device[MAXLEN_TTYSUFFIX+128] = {0, };
	int ttyd_pid = -1;

	sprintf(device, "%s%s", VTTY_MASTER_PREFIX, ttysuffix);
	sprintf(cmd, "fuser %s", device);
	fp = popen(cmd, "r");
	if(fp)
	{
		char buff[MAXLEN_TTYSUFFIX+128] = {0, };
		char *p = NULL;
		if(fgets(buff, sizeof(buff), fp) && (p = strstr(buff, device)) != NULL)
			sscanf(buff, "%s%d", device, &ttyd_pid);

		if (ttyd_pid == -1) ttyd_pid = atoi(buff);//Fix for Debian 3.1
		fclose(fp);
	}
	return ttyd_pid;
}

int check_driver()
{
#define VPTY_DRIVER_NAME "sena_vpty"
	FILE *fp;
	fp = popen("cat /proc/modules", "r");
	if(fp)
	{
		int binstalled = 0;
		char buff[512] = {0, };
		while(fgets(buff, sizeof(buff), fp))
		{
			if(strstr(buff, VPTY_DRIVER_NAME))
			{
				binstalled = 1; 
				break;
			}
		}
		fclose(fp);
		if(binstalled)
			return 0;
	}
	return -1;
}

int load_driver()
{
	char cmd[] = "insmod /usr/local/vpty/sena_vpty.o";
	if(check_driver() < 0)
	{
		printf("loading pseudo-tty driver module for vtty...\r\n");
		system(cmd);
	}
	else
		return 0;
	return check_driver();
}

int kill_ttyd(char *ttysuffix)
{
	char cmd[128] = {0, };
	int pid = get_ttyd_pid(ttysuffix);
	
	if(pid <= 0) // Fix the bug in removing ttyd if not running - JOJ 
		return -1;
	sprintf(cmd, "kill %d", pid);
	system(cmd);
	usleep(500000);

	// check again
	if(get_ttyd_pid(ttysuffix) < 0)
		return 0;
	else
		return -1;
}

int run_ttyd(struct _vtty_db_type_t data)
{
	char ttydcmd[1024] = {0, };
	char portsetting[32] = {0, };
	// execute ttyd
	get_port_setting(portsetting, data.uart);
	if (toffering) // Enable Telnet Options Offering - JOJ
	  sprintf(ttydcmd, "%s -d %s%s %s %s %d", VTTY_TTYD_PATH, VTTY_MASTER_PREFIX, data.ttysuffix, portsetting, data.target_ip, data.target_port);
	else // Disable elnet Options Offering - JOJ
	  sprintf(ttydcmd, "%s -t -d %s%s %s %s %d", VTTY_TTYD_PATH, VTTY_MASTER_PREFIX, data.ttysuffix, portsetting, data.target_ip, data.target_port); 
	system(ttydcmd);
	usleep(500000);
	if(get_ttyd_pid(data.ttysuffix) < 0)
		return -1;
	else
		return 0;
}

int get_port_setting(char *portsetting, struct _vtty_uart_type_t uart)
{
	int i=0;
	sprintf(portsetting, "-b %d -p ", uart.baudrate);
	while(pset_db[i].symbol) {
		if(uart.databits == pset_db[i].val) {
			strcat(portsetting, pset_db[i].symbol); break;
		} i++;
	}
	i=0;
	while(pset_parity[i].symbol) {
		if(uart.parity == pset_parity[i].val) {
			strcat(portsetting, pset_parity[i].symbol); break;
		} i++;
	}
	i=0;
	while(pset_fc[i].symbol) {
		if(uart.flowctrl == pset_fc[i].val) {
			strcat(portsetting, pset_fc[i].symbol); break;
		} i++;
	}
}

#define MAKE_DEV_T(major, minor) (major<<8)+minor

void do_vtty(struct _vtty_command_set *cmd, char *progname)
{
	char devname[256] = {0, };
	dev_t dev;
	struct _vtty_db_type_t data = {0, };
	int minor;
	int net_stat = 0;
	struct stat astat;

	// make vtty db storage directory if not exists
	if(stat(VTTY_DB_PATH, &astat) < 0) 
		mkdir(VTTY_DB_PATH, 0755);

	if(vtty_db_wopen() < 0) { fprintf(stderr, "Error, vtty db open error!\r\n"); return; }
	switch(cmd->command)
	{
		case VTTY_CMD_ADD:
			if(vtty_db_count() >= MAX_DB_ENTRY)
			{
				fprintf(stderr, "Error, device add limit exceeded. Max. 255 devices\r\n");
				vtty_db_close();
				return;
			}
			// open db and check whether the key exists
			if(vtty_db_exists(cmd->type.ttysuffix))
			{
				fprintf(stderr, "Error, %s - already exists in the list\r\n", cmd->type.ttysuffix);
				vtty_db_close();
				return;
			}
			// get available minor
			minor = vtty_db_get_available_minor();
			if(minor < 0)
			{
				fprintf(stderr, "Error, there is no available device minor!\r\n");
				vtty_db_close();
				return;
			}
			// check and load driver
			if(load_driver() < 0)
			{
				fprintf(stderr, "Error, vpty driver is not properly loaded. Nothing has been added!\r\n");
				vtty_db_close();
				return;
			}
			// make master node 
   			dev = MAKE_DEV_T(VTTY_MAJOR_MASTER, minor);
			sprintf(devname, "%s%s", VTTY_MASTER_PREFIX, cmd->type.ttysuffix);
			if(mknod(devname, S_IFCHR | 0644 , dev) < 0)
			{
				if(errno == EEXIST)
					fprintf(stderr, "Error, a device with its name %s%s already exists.\r\n", VTTY_MASTER_PREFIX, cmd->type.ttysuffix);
				else
					fprintf(stderr, "Error, couldn't make the master node!\r\n");
				vtty_db_close();
				return;
			}
			// make slave node
   			dev = MAKE_DEV_T(VTTY_MAJOR_SLAVE, minor);
			sprintf(devname, "%s%s", VTTY_SLAVE_PREFIX, cmd->type.ttysuffix);
			if(mknod(devname, S_IFCHR | 0644 , dev) < 0)
			{
				if(errno == EEXIST)
					fprintf(stderr, "Error, a device with its name %s%s already exists.\r\n", VTTY_SLAVE_PREFIX, cmd->type.ttysuffix);
				else
					fprintf(stderr, "Error, couldn't make the slave node!\r\n");
				sprintf(devname, "%s%s", VTTY_MASTER_PREFIX, cmd->type.ttysuffix);
				unlink(devname);
				vtty_db_close();
				return;
			}
			memcpy(&data, &cmd->type, sizeof(struct _vtty_db_type_t));
			data.minor = minor;
			data.benable = STATUS_ENABLED;
			// execute ttyd
			if(run_ttyd(cmd->type) < 0)
			{
				fprintf(stderr, "Error, %s - failed to run vtty. Nothing has been added!\r\n", cmd->type.ttysuffix);
				sprintf(devname, "%s%s", VTTY_MASTER_PREFIX, cmd->type.ttysuffix);
				unlink(devname);
				sprintf(devname, "%s%s", VTTY_SLAVE_PREFIX, cmd->type.ttysuffix);
				unlink(devname);
				vtty_db_close();
				return;
			}
			vtty_db_write((void *)&data);
			fprintf(stderr, "%s has been added successfully!\r\n", cmd->type.ttysuffix);
			break;
		case VTTY_CMD_ENABLE:
			if(!vtty_db_exists(cmd->type.ttysuffix))
			{
				fprintf(stderr, "Error, %s - does not exist\r\n", cmd->type.ttysuffix);
				vtty_db_close();
				return;
			}
			vtty_db_get(cmd->type.ttysuffix, (void *)&data);
			data.benable = STATUS_ENABLED;
			// check and load driver
			if(load_driver() < 0)
			{
				fprintf(stderr, "Error, vpty driver is not properly loaded. The node has not been enabled!\r\n");
				vtty_db_close();
				return;
			}
			// execute ttyd
			if(run_ttyd(data) < 0)
			{
				fprintf(stderr, "Error, %s - failed to run vtty. The node has not been enabled!\r\n", cmd->type.ttysuffix);
				vtty_db_close();
				return;
			}
			vtty_db_write((void *)&data);
			fprintf(stderr, "%s has been enabled successfully!\r\n", cmd->type.ttysuffix);
			break;
		case VTTY_CMD_DISABLE:
			if(!vtty_db_exists(cmd->type.ttysuffix))
			{
				fprintf(stderr, "Error, %s - does not exist\r\n", cmd->type.ttysuffix);
				vtty_db_close();
				return;
			}
			vtty_db_get(cmd->type.ttysuffix, (void *)&data);
			data.benable = STATUS_DISABLED;
			kill_ttyd(cmd->type.ttysuffix); 
			vtty_db_write((void *)&data);
			fprintf(stderr, "%s has been disabled successfully!\r\n", cmd->type.ttysuffix);
			break;
		case VTTY_CMD_REMOVE:
			if(!vtty_db_exists(cmd->type.ttysuffix))
			{
				fprintf(stderr, "Error, %s - does not exist\r\n", cmd->type.ttysuffix);
				vtty_db_close();
				return;
			}
			kill_ttyd(cmd->type.ttysuffix);
			sprintf(devname, "%s%s", VTTY_MASTER_PREFIX, cmd->type.ttysuffix);
			unlink(devname);
			sprintf(devname, "%s%s", VTTY_SLAVE_PREFIX, cmd->type.ttysuffix);
			unlink(devname);
			vtty_db_delete(cmd->type.ttysuffix);
			fprintf(stderr, "%s has been removed successfully!\r\n", cmd->type.ttysuffix);
			break;
		case VTTY_CMD_STATUS:
			printf("\r\n");
			printf("DEVICE TTYNAME                 IP              PORT  SERIAL      STATUS      \r\n");
			printf("-----------------------------------------------------------------------------\r\n");
			if(strlen(cmd->type.ttysuffix) > 0)
			{
				if(!vtty_db_exists(cmd->type.ttysuffix))
				{
					fprintf(stderr, "Error, %s - does not exist\r\n", cmd->type.ttysuffix);
					vtty_db_close();
					printf("-----------------------------------------------------------------------------\r\n");
					return;
				}
				vtty_db_get(cmd->type.ttysuffix, (void *)&data);
				net_stat = check_network(data.target_ip, data.target_port);
				print_status(&data, net_stat);
			}
			else
			{
				void *p = NULL;
				int i=0;
				while((i = vtty_db_get_next_by_minornum(i, (void *)&data)) >= 0)
				{
					net_stat = check_network(data.target_ip, data.target_port);
					print_status(&data, net_stat);
					p = &data;
					i++;
				}
			}
			printf("-----------------------------------------------------------------------------\r\n");
			break;
		case VTTY_CMD_INIT:
			{
				void *p = NULL;
				int i=0;
				
				printf("Initializing vtty ...\r\n");
				// load kernel module. check and load driver
				if(load_driver() < 0)
				{
					fprintf(stderr, "Error, vpty driver loading failed!\r\n");
					vtty_db_close();
					return;
				}

				// auto start registered ttyd
				while((i = vtty_db_get_next_by_minornum(i, (void *)&data)) >= 0)
				{
					if(data.benable == STATUS_ENABLED)
					{
						if(get_ttyd_pid(data.ttysuffix) < 0)
							run_ttyd(data);
					}
					p = &data;
					i++;
				}
			}
			break;
		default:
			break;
	}
	vtty_db_close();
}

int main(int argc, char *argv[])
{
	struct _vtty_command_set cmd;	

	if(argc < 2)
	{
		print_usage(argv[0]);
		exit(1);
	}

	memset(&cmd, 0, sizeof(struct _vtty_command_set));	
	cmd.type.uart.baudrate = B9600;
	cmd.type.uart.parity = PARITY_NO;
	cmd.type.uart.databits = DB8;
	cmd.type.uart.flowctrl = FC_NOHW;

	if(parse_options(argc, &argv[0], &cmd) < 0)
		exit(1);

	do_vtty(&cmd, argv[0]);

	exit(0);
}


